@@ -1,13 +1,10 @@
-#![feature(test)]
+use criterion::{criterion_group, criterion_main, Bencher, Criterion};
 
-extern crate test;
 use pyo3::prelude::*;
 use pyo3::types::PySet;
 use std::collections::{BTreeSet, HashSet};
-use test::Bencher;
 
-#[bench]
-fn iter_set(b: &mut Bencher) {
+fn iter_set(b: &mut Bencher<'_>) {
     let gil = Python::acquire_gil();
     let py = gil.python();
     const LEN: usize = 100_000;
@@ -21,8 +18,7 @@ fn iter_set(b: &mut Bencher) {
     });
 }
 
-#[bench]
-fn extract_hashset(b: &mut Bencher) {
+fn extract_hashset(b: &mut Bencher<'_>) {
     let gil = Python::acquire_gil();
     let py = gil.python();
     const LEN: usize = 100_000;
@@ -30,8 +26,7 @@ fn extract_hashset(b: &mut Bencher) {
     b.iter(|| HashSet::<u64>::extract(set));
 }
 
-#[bench]
-fn extract_btreeset(b: &mut Bencher) {
+fn extract_btreeset(b: &mut Bencher<'_>) {
     let gil = Python::acquire_gil();
     let py = gil.python();
     const LEN: usize = 100_000;
@@ -39,12 +34,23 @@ fn extract_btreeset(b: &mut Bencher) {
     b.iter(|| BTreeSet::<u64>::extract(set));
 }
 
-#[bench]
 #[cfg(feature = "hashbrown")]
-fn extract_hashbrown_set(b: &mut Bencher) {
+fn extract_hashbrown_set(b: &mut Bencher<'_>) {
     let gil = Python::acquire_gil();
     let py = gil.python();
     const LEN: usize = 100_000;
     let set = PySet::new(py, &(0..LEN).collect::<Vec<_>>()).unwrap();
     b.iter(|| hashbrown::HashSet::<u64>::extract(set));
 }
+
+fn criterion_benchmark(c: &mut Criterion) {
+    c.bench_function("iter_set", iter_set);
+    c.bench_function("extract_hashset", extract_hashset);
+    c.bench_function("extract_btreeset", extract_btreeset);
+
+    #[cfg(feature = "hashbrown")]
+    c.bench_function("extract_hashbrown_set", extract_hashbrown_set);
+}
+
+criterion_group!(benches, criterion_benchmark);
+criterion_main!(benches);
